@@ -13,7 +13,7 @@
 ##############################################################################
 """Interfaces for the publisher.
 
-$Id: __init__.py 105429 2009-11-02 07:47:51Z ctheune $
+$Id: __init__.py 106563 2009-12-15 19:48:23Z hannosch $
 """
 __docformat__ = "reStructuredText"
 
@@ -465,6 +465,20 @@ class IRequest(IPublisherRequest, IPublicationRequest, IApplicationRequest):
     """
 
 
+class IEndRequestEvent(Interface):
+    """An event which gets sent when the publication is ended"""
+
+
+class EndRequestEvent(object):
+    """An event which gets sent when the publication is ended"""
+
+    implements(IEndRequestEvent)
+
+    def __init__(self, ob, request):
+        self.object = ob
+        self.request = request
+
+
 class ISkinType(IInterface):
     """Base interface for skin types."""
 
